/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomElement.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomElement.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateIntGdomElement.h"
#include "gdom/PrivateIntGdomHTMLElement.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLElement::GDOMHTMLElement(PassRefPtr<WebCore::HTMLElement> impl)
    : GDOMElement(impl)
{
}

void
gdom_html_element_blur (GdomHTMLElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLElement * item = (gdom_html_element_get_private(thiz));
    g_return_if_fail (item);
    item->blur();

}

void
gdom_html_element_focus (GdomHTMLElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLElement * item = (gdom_html_element_get_private(thiz));
    g_return_if_fail (item);
    item->focus();

}

GdomElement *
gdom_html_element_insert_adjacent_element (GdomHTMLElement *thiz, gchar * where, GdomElement * element)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLElement * item = (gdom_html_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (where, 0);
    g_return_val_if_fail (element, 0);
    WebCore::String _g_where = WebCore::String::fromUTF8(where);
    WebCore::Element * _g_element = (gdom_element_get_private(element));
    g_return_val_if_fail (_g_element, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Element> g_res = WTF::getPtr(item->insertAdjacentElement(_g_where, _g_element, ec));
    GdomElement * res = static_cast<GdomElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_html_element_insert_adjacent_html (GdomHTMLElement *thiz, gchar * where, gchar * html)
{
    g_return_if_fail (thiz);
    WebCore::HTMLElement * item = (gdom_html_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (where);
    g_return_if_fail (html);
    WebCore::String _g_where = WebCore::String::fromUTF8(where);
    WebCore::String _g_html = WebCore::String::fromUTF8(html);
    WebCore::ExceptionCode ec = 0;
    item->insertAdjacentHTML(_g_where, _g_html, ec);

}

void
gdom_html_element_insert_adjacent_text (GdomHTMLElement *thiz, gchar * where, gchar * text)
{
    g_return_if_fail (thiz);
    WebCore::HTMLElement * item = (gdom_html_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (where);
    g_return_if_fail (text);
    WebCore::String _g_where = WebCore::String::fromUTF8(where);
    WebCore::String _g_text = WebCore::String::fromUTF8(text);
    WebCore::ExceptionCode ec = 0;
    item->insertAdjacentText(_g_where, _g_text, ec);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLElement.h"
#include <gdom/GdomHTMLElement.h>
#include <gdom/PrivateGdomHTMLElement.h>
#include <gdom/PrivateIntGdomHTMLElement.h>
#include <gdom/GdomHTMLElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLElement, gdom_html_element, GDOM_TYPE_ELEMENT);

/*
struct _GdomHTMLElementPrivate {
    class WebCore::HTMLElement *item;
};

#define GDOM_HTML_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_ELEMENT, GdomHTMLElementPrivate))
*/
G_END_DECLS
WebCore::HTMLElement *
gdom_html_element_get_private(GdomHTMLElement *request)
{
    GDOMHTMLElement *res;
    res = static_cast<GDOMHTMLElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ID,
    PROP_TITLE,
    PROP_LANG,
    PROP_DIR,
    PROP_CLASS_NAME,
    PROP_TAB_INDEX,
    PROP_INNER_HTML,
    PROP_INNER_TEXT,
    PROP_OUTER_HTML,
    PROP_OUTER_TEXT,
    PROP_CHILDREN,
    PROP_CONTENT_EDITABLE,
    PROP_IS_CONTENT_EDITABLE,
};


static void gdom_html_element_finalize(GObject* object)
{
    /*GdomHTMLElement* request = GDOM_HTML_ELEMENT(object);*/
    /*GdomHTMLElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_element_parent_class)->finalize(object);
}

static void gdom_html_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLElement *self = GDOM_HTML_ELEMENT(object);
    WebCore::HTMLElement * gself = (gdom_html_element_get_private(self));
    switch (prop_id) {
     case PROP_ID:
    {
         gself->setId(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TITLE:
    {
         gself->setTitle(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_LANG:
    {
         gself->setLang(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_DIR:
    {
         gself->setDir(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_CLASS_NAME:
    {
         gself->setClassName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TAB_INDEX:
    {
         gself->setTabIndex((g_value_get_long (value)) );
         break;
    }
     case PROP_INNER_HTML:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setInnerHTML(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_INNER_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setInnerText(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_OUTER_HTML:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setOuterHTML(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_OUTER_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setOuterText(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_CONTENT_EDITABLE:
    {
         gself->setContentEditable(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLElement *self = GDOM_HTML_ELEMENT(object);
    WebCore::HTMLElement * gself = (gdom_html_element_get_private(self));
    switch (prop_id) {
    case PROP_ID:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->id()));
         break;
    }
    case PROP_TITLE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->title()));
         break;
    }
    case PROP_LANG:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->lang()));
         break;
    }
    case PROP_DIR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->dir()));
         break;
    }
    case PROP_CLASS_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->className()));
         break;
    }
    case PROP_TAB_INDEX:
    {
         g_value_set_long (value, gself->tabIndex());
         break;
    }
    case PROP_INNER_HTML:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->innerHTML()));
         break;
    }
    case PROP_INNER_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->innerText()));
         break;
    }
    case PROP_OUTER_HTML:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->outerHTML()));
         break;
    }
    case PROP_OUTER_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->outerText()));
         break;
    }
    case PROP_CHILDREN:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->children();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CONTENT_EDITABLE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->contentEditable()));
         break;
    }
    case PROP_IS_CONTENT_EDITABLE:
    {
         g_value_set_boolean (value, gself->isContentEditable());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_element_class_init(GdomHTMLElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_element_finalize;
    g_klass->set_property = gdom_html_element__set_property_fn;
    g_klass->get_property = gdom_html_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_ID,
            g_param_spec_string ("id", /* name */
                "html_element_id", /* short description */
                "read-write  gchar * HTMLElement.id", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TITLE,
            g_param_spec_string ("title", /* name */
                "html_element_title", /* short description */
                "read-write  gchar * HTMLElement.title", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LANG,
            g_param_spec_string ("lang", /* name */
                "html_element_lang", /* short description */
                "read-write  gchar * HTMLElement.lang", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DIR,
            g_param_spec_string ("dir", /* name */
                "html_element_dir", /* short description */
                "read-write  gchar * HTMLElement.dir", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CLASS_NAME,
            g_param_spec_string ("class_name", /* name */
                "html_element_class_name", /* short description */
                "read-write  gchar * HTMLElement.class_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TAB_INDEX,
            g_param_spec_long ("tab_index", /* name */
                "html_element_tab_index", /* short description */
                "read-write  glong HTMLElement.tab_index", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_INNER_HTML,
            g_param_spec_string ("inner_html", /* name */
                "html_element_inner_html", /* short description */
                "read-write  gchar * HTMLElement.inner_html", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_INNER_TEXT,
            g_param_spec_string ("inner_text", /* name */
                "html_element_inner_text", /* short description */
                "read-write  gchar * HTMLElement.inner_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OUTER_HTML,
            g_param_spec_string ("outer_html", /* name */
                "html_element_outer_html", /* short description */
                "read-write  gchar * HTMLElement.outer_html", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OUTER_TEXT,
            g_param_spec_string ("outer_text", /* name */
                "html_element_outer_text", /* short description */
                "read-write  gchar * HTMLElement.outer_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CHILDREN,
            g_param_spec_object ("children", /* name */
                "html_element_children", /* short description */
                "read-only  GdomHTMLCollection * HTMLElement.children", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CONTENT_EDITABLE,
            g_param_spec_string ("content_editable", /* name */
                "html_element_content_editable", /* short description */
                "read-write  gchar * HTMLElement.content_editable", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_IS_CONTENT_EDITABLE,
            g_param_spec_boolean ("is_content_editable", /* name */
                "html_element_is_content_editable", /* short description */
                "read-only  gboolean HTMLElement.is_content_editable", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLElementPrivate));*/
}

static void gdom_html_element_init(GdomHTMLElement* request)
{
    /*GdomHTMLElementPrivate* priv = GDOM_HTML_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

